<?php

	class Icwoorok2Controller
	{
		private static $bInitiated = false;

		public static function init()
		{
			if(self::$bInitiated)
			{
				return;
			}

			// Load methods
			add_action('woocommerce_payment_gateways', array(__CLASS__, 'getGateways'));

			// Load settings
			add_filter('woocommerce_payment_gateways_settings', array(__CLASS__, 'addPspSettings'));

			// Set hooks for the return and webhook call
			add_action('woocommerce_api_wc_omnikassa_gateway_return', array(__CLASS__, 'doReturn'));
			add_action('woocommerce_api_wc_omnikassa_gateway_notify', array(__CLASS__, 'doNotify'));

			// Mark plugin initiated
			self::$bInitiated = true;
		}


		public static function getGateways($aDefaultGateways)
		{
			// Load our gateways
			$sFolderPath = ICWOOROK_ROOT_PATH . DIRECTORY_SEPARATOR . 'gateways';
			icwoorok2_loadFilesFromFolder($sFolderPath);

			$aPaymentGateways = array(
				'icwoorok2_bancontact',
				'icwoorok2_ideal',
				'icwoorok2_afterpay',
				'icwoorok2_creditcard',
				'icwoorok2_paypal',
			);

			$aDefaultGateways = array_merge($aDefaultGateways, $aPaymentGateways);

			return $aDefaultGateways;
		}

		public static function addPspSettings($aSettings)
		{
			$sWebhookUrl = add_query_arg('wc-api', 'Wc_Omnikassa_Gateway_Notify', home_url('/'));

			$aAddedSettings = array();
			$aAddedSettings[] = array(
				'name' => __('Rabo OmniKassa 2.0 - Merchant settings', 'ic-woo-rabo-omnikassa-2'),
				'type' => 'title',
				'desc' => __('IMPORTANT: To process transactions properly, please enter the following webhook URL on your Rabo OmniKassa dashboard:', 'ic-woo-rabo-omnikassa-2') . ' <a href="' . $sWebhookUrl . '">' . $sWebhookUrl . '</a></p>',
			);
			$aAddedSettings[] = array(
				'name' => __('Refresh Token', 'ic-woo-rabo-omnikassa-2'),
				'type' => 'textarea',
				'css' => 'height:200px;font-size:12px;',
				'desc' => '<p>' . __('The Refresh token can be found on the Rabo OmniKassa 2.0 dashboard.', 'ic-woo-rabo-omnikassa-2') . '</p>',
				'id' => 'icwoorok2_refresh_token',
			);
			$aAddedSettings[] = array(
				'name' => __('Signing Key', 'ic-woo-rabo-omnikassa-2'),
				'type' => 'text',
				'css' => 'font-size:12px;',
				'desc' => '<p>' . __('The Signing Key token can be found on the Rabo OmniKassa 2.0 dashboard.', 'ic-woo-rabo-omnikassa-2') . '</p>',
				'id' => 'icwoorok2_signing_key',
			);
			$aAddedSettings[] = array(
				'name' => __('Sandbox environment', 'ic-woo-rabo-omnikassa-2'),
				'type' => 'checkbox',
				'label' => __('Enable the Sandbox mode', 'ic-woo-rabo-omnikassa-2'),
				'default' => 'yes',
				'desc' => '<p>' . __('Please make sure you enter the sandbox credentials if checked.', 'ic-woo-rabo-omnikassa-2') . '</p>',
				'id' => 'icwoorok2_sandbox',
			);
			$aAddedSettings[] = array(
				'name' => __('Bank selection in checkout', 'ic-woo-rabo-omnikassa-2'),
				'type' => 'select',
				'label' => __('Show bank selection', 'ic-woo-rabo-omnikassa-2'),
				'default' => '',
				'options' => array(
					'0' => __('Dont show', 'ic-woo-rabo-omnikassa-2'),
					'1' => __('Show as select field', 'ic-woo-rabo-omnikassa-2'),
					'2' => __('Show as radio buttons', 'ic-woo-rabo-omnikassa-2')
				),
				'desc' => '<p>' . __('Display iDEAL bank selection in the checkout.', 'ic-woo-rabo-omnikassa-2') . '</p>',
				'id' => 'icwoorok2_show_issuers',
			);


			$aAddedSettings[] = array(
				'type' => 'sectionend',
				'id' => 'icwoorok2_settings',
			);

			return array_merge($aAddedSettings, $aSettings);
		}

		public static function doReturn()
		{
			global $woocommerce;

			if(empty($_GET['status']) && empty($_GET['order_id']) && empty($_GET['signature']) && empty($_GET['real_id']))
			{
				wp_redirect($woocommerce->cart->get_cart_url());
			}
			else
			{
				$sMerchantOrderId = sanitize_text_field($_GET['order_id']);
				$sOmniKassaStatus = sanitize_text_field($_GET['status']);
				$sOmniKassaSignature = sanitize_text_field($_GET['signature']);
				$sOrderId = sanitize_text_field($_GET['real_id']);
				$bUtmOverride = array_key_exists('utm_nooverride', $_GET);

				$oOrder = wc_get_order($sOrderId);

				$sPaymentMethod = $oOrder->get_payment_method_title();

				$sHashString = $sMerchantOrderId . ',' . $sOmniKassaStatus;
				$sHash = hash_hmac('sha512', $sHashString, base64_decode(get_option('icwoorok2_signing_key')));

				if(strcmp($sOmniKassaSignature, $sHash) === 0)
				{
					if(strcmp($sOmniKassaStatus, 'COMPLETED') === 0)
					{
						$sReturnUrl = $oOrder->get_checkout_order_received_url();

						$aPaymentStatuses = wc_get_is_paid_statuses();
						$aPaymentStatuses[] = 'refunded';

						if(!in_array($oOrder->get_status(), $aPaymentStatuses))
						{
							$oOrder->add_order_note(__('Status received from Customer Return: ', 'ic-woo-rabo-omnikassa-2') . $sOmniKassaStatus . '. ' . __('Order updated, check OmniKassa dashboard for status before sending products', 'ic-woo-rabo-omnikassa-2') . '. ' . __('Payment-method: ', 'ic-woo-rabo-omnikassa-2') . $sPaymentMethod);

							$oOrder->payment_complete();
						}
						else
						{
							$oOrder->add_order_note(__('Status received from Customer Return:', 'ic-woo-rabo-omnikassa-2') . $sOmniKassaStatus . '. ' . __('Payment status already received: Order status not updated, check OmniKassa dashboard for status', 'ic-woo-rabo-omnikassa-2') . '. ' . __('Payment-method: ', 'ic-woo-rabo-omnikassa-2') . $sPaymentMethod);
						}

						wp_redirect($sReturnUrl . ($bUtmOverride ? '&utm_nooverride=1' : ''));
					}
					else
					{
						$oOrder->add_order_note(__('Status received from Customer Return:', 'ic-woo-rabo-omnikassa-2') . $sOmniKassaStatus . '. ' . __('Order not updated, check OmniKassa dashboard for status', 'ic-woo-rabo-omnikassa-2') . '. ' . __('Payment-method: ', 'ic-woo-rabo-omnikassa-2') . $sPaymentMethod);
						wp_redirect(wc_get_cart_url() . ($bUtmOverride ? '&utm_nooverride=1' : ''));
					}
				}
				else
				{
					wp_redirect(wc_get_cart_url() . ($bUtmOverride ? '&utm_nooverride=1' : ''));
				}
			}
		}

		public static function doNotify($return = false)
		{
			global $woocommerce;
			$sHtml = '';

			// Build the notify from the Rabobank OmniKassa
			$sJsonData = @file_get_contents('php://input');

			if(empty($sJsonData))
			{
				http_response_code(400);
				$sHtml .= '<p>Invalid notification call.</p>';
			}
			else
			{
				$aPostData = json_decode($sJsonData, true);

				if(!empty($aPostData['authentication']) && !empty($aPostData['expiry']) && !empty($aPostData['eventName']) && !empty($aPostData['poiId']) && !empty($aPostData['signature']))
				{
					$sAuthToken = sanitize_text_field($aPostData['authentication']);
					$sExpiryString = sanitize_text_field($aPostData['expiry']);
					$sEventnameString = sanitize_text_field($aPostData['eventName']);
					$sPoiIdString = sanitize_text_field($aPostData['poiId']);
					$sSignatureString = sanitize_text_field($aPostData['signature']);

					// Check the signature that was sent with the data
					$sHashString = $sAuthToken . ',' . $sExpiryString . ',' . $sEventnameString . ',' . $sPoiIdString;

					$sHash = hash_hmac('sha512', $sHashString, base64_decode(get_option('icwoorok2_signing_key')));

					if(strcmp($sSignatureString, $sHash) === 0)
					{
						$sTestMode = get_option('icwoorok2_sandbox');
						$bSandboxMode = (strcmp($sTestMode, 'yes') === 0);

						$sApiUrl = 'https://betalen.rabobank.nl/omnikassa-api' . ($bSandboxMode ? '-sandbox' : '') . '/order/server/api/v2/events/results/merchant.order.status.changed';

						$aResult = array();

						do
						{
							$aArguments = array();
							$aArguments['body'] = '';
							$aArguments['timeout'] = '30';
							$aArguments['redirection'] = '5';
							$aArguments['httpversion'] = '1.1';
							$aArguments['blocking'] = true;
							$aArguments['headers'] = array('Expect' => '', 'Authorization'=> 'Bearer ' . $sAuthToken);
							$aArguments['cookies'] = array();

							$oResponse = wp_remote_get($sApiUrl, $aArguments);
							$sResponse = wp_remote_retrieve_body($oResponse);

							// error_log($sResponse);

							$aResult = json_decode($sResponse, true);
							if(!empty($aResult) && sizeof($aResult))
							{
								$sSignature = $aResult['signature'];
								$bMoreOrders = $aResult['moreOrderResultsAvailable'];
								
								if($bMoreOrders)
								{
									$sMoreOrdersAvailable = 'true';
								}
								else
								{
									$sMoreOrdersAvailable = 'false';

								}

								$sHashString = $sMoreOrdersAvailable . ',';

								// Validate total signature
								foreach($aResult['orderResults'] as $aRokResult)
								{
									// Setup total hash string
									$sTransactionString = $aRokResult['merchantOrderId'] . ',' . $aRokResult['omnikassaOrderId'] . ',' . $aRokResult['poiId'] . ',' . $aRokResult['orderStatus'] . ',' . $aRokResult['orderStatusDateTime'] . ',' . $aRokResult['errorCode'] . ',' . $aRokResult['paidAmount']['currency'] . ',' . $aRokResult['paidAmount']['amount'] . ',' . $aRokResult['totalAmount']['currency'] . ',' . $aRokResult['totalAmount']['amount'];


									// Add all transactions
									foreach($aRokResult['transactions'] as $aTransaction)
									{
										$sTransactionString .= ',' . $aTransaction['id'] . ',' . $aTransaction['paymentBrand'] . ',' . $aTransaction['type'] . ',' . $aTransaction['status'] . ',' . $aTransaction['amount']['currency'] . ',' . $aTransaction['amount']['amount']  . ',' . $aTransaction['confirmedAmount']['currency'] . ',' . $aTransaction['confirmedAmount']['amount'] . ',' . $aTransaction['startTime'] . ',' . $aTransaction['lastUpdateTime'];
									}

									$sHashString .= $sTransactionString . ',';
								}
																
								// Cut off last comma
								$sHashString = substr($sHashString, 0, -1);
								$sHash = hash_hmac('sha512', $sHashString, base64_decode(get_option('icwoorok2_signing_key')));

								if(strcmp($sSignature, $sHash) === 0)
								{
									foreach($aResult['orderResults'] as $aRokResult)
									{
										$sMerchantOrderId = $aRokResult['merchantOrderId'];
										$sOmniKassaStatus = $aRokResult['orderStatus'];
										$sTransactionId = icwoorok2_getTransactionId($aRokResult['transactions']);

										// Get order by Merchant Order ID
										$oOrder = wc_get_order($sMerchantOrderId);
										$sOrderStatus = $oOrder->get_status();

										$sMessage = __('Status received from Rabo OmniKassa Webhook: ', 'ic-woo-rabo-omnikassa-2') . $sOmniKassaStatus . __(' for transaction ID:', 'ic-woo-rabo-omnikassa-2', 'ic-woo-rabo-omnikassa-2') . '<br>' . $sTransactionId;

										if(in_array($sOrderStatus, array('pending', 'failed', 'cancelled')))
										{
											if(strcmp($sOmniKassaStatus, 'COMPLETED') === 0)
											{
												$aPaymentStatuses = wc_get_is_paid_statuses();
												$aPaymentStatuses[] = 'refunded';

												if(!in_array($sOrderStatus, $aPaymentStatuses))
												{
													$sMessage .= '<br>' . __('Update order status: Payment Completed', 'ic-woo-rabo-omnikassa-2');
													$oOrder->add_order_note($sMessage);
													$oOrder->payment_complete($sTransactionId);
												}
												else
												{
													$sMessage .= '<br>' . __('Order status not updated: Payment status already received', 'ic-woo-rabo-omnikassa-2');
													$oOrder->add_order_note($sMessage);
												}
											}
											elseif(in_array($sOmniKassaStatus, array('CANCELLED', 'EXPIRED')))
											{
												// Check if WooCommerce cancels automaticly for the stock management
												$iHoldStockMinutes = get_option('woocommerce_hold_stock_minutes');

												if(!empty($iHoldStockMinutes) && ($iHoldStockMinutes > 0))
												{
													//Happens automaticly, we dont need to do anything
													$sMessage .= __('<br>Payment is cancelled or expired, but will be cancelled automaticly by WooCommerce.', 'ic-woo-rabo-omnikassa-2');
													$oOrder->add_order_note($sMessage);
												}
												else
												{
													if(strcmp($sOmniKassaStatus, 'EXPIRED') === 0)
													{
														$sMessage .= __('<br>Update order status: Failed.', 'ic-woo-rabo-omnikassa-2');
														$oOrder->add_order_note($sMessage);

														$oOrder->update_status('failed');
													}
													elseif(strcmp($sOmniKassaStatus, 'CANCELLED') === 0)
													{
														$sMessage .= __('<br>Update order status: Cancelled', 'ic-woo-rabo-omnikassa-2');
														$oOrder->add_order_note($sMessage);

														$oOrder->update_status('cancelled');
													}
													else
													{
														// Possibly another status to be implemented?
														$oOrder->add_order_note($sMessage);
													}
												}
											}
											else
											{
												// No final status
												$sMessage .= __('<br>No Final status has been found.', 'ic-woo-rabo-omnikassa-2');
												$oOrder->add_order_note($sMessage);
											}
										}
										else // pending
										{
											if($sTransactionId && (strcmp($sOmniKassaStatus, 'COMPLETED') === 0))
											{
												$sMessage .= __('<br>Transaction ID updated.', 'ic-woo-rabo-omnikassa-2');
												$oOrder->set_transaction_id($sTransactionId);
												$oOrder->save();
											}
											else
											{
												$sMessage .= __('<br>Order doesnt have the correct status to be changed by the payment method.', 'ic-woo-rabo-omnikassa-2');
											}

											$oOrder->add_order_note($sMessage);
											// This shouldn't happen
										}
									}
								}
							}
						}
						while($bMoreOrders);
					}
					else {
						
						http_response_code(403);
					}
				}
				else
				{
					$sHtml .= '<p>Invalid notification call.</p>';
				}
			}

			echo $sHtml;
			exit;
		}
	}


?>