<?php

	class icwoorok2_abstract extends WC_Payment_Gateway
	{
		public function __construct()
		{
			$this->id = $this->getPaymentCode();
			$this->icon = $this->getIcon();
			$this->has_fields = true;
			$this->method_title = $this->getPaymentName() . $this->getLabel();
			$this->method_description = sprintf(__('Enable this method to receive transactions with Rabo OmniKassa - %s ', 'ic-woo-rabo-omnikassa-2'), $this->getPaymentName());

			$this->supports = array('products');

			$this->init_form_fields();
			$this->init_settings();

			$this->title = $this->get_option('title');

			add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
		}

		public function getAccessToken()
		{
			$sCacheFile = false;
			$sCachePath = ICWOOROK_ROOT_PATH . 'temp' . DIRECTORY_SEPARATOR . 'cache' . DIRECTORY_SEPARATOR;

			// Used cached access token?
			// if(!$this->getSandboxMode() && $sCachePath)
			if($sCachePath) //$sCachePath)
			{
				$sStoreHost = md5($_SERVER['SERVER_NAME']);
				$sCacheFile = $sCachePath . 'token.' . $sStoreHost . '.cache';

				if(!file_exists($sCacheFile))
				{
					// Attempt to create cache file
					if(@touch($sCacheFile))
					{
						@chmod($sCacheFile, 0777);
					}
				}
				elseif(is_readable($sCacheFile) && is_writable($sCacheFile))
				{
					// Read data from cache file
					if($sData = file_get_contents($sCacheFile))
					{
						$aToken = json_decode($sData, true);

						// Get current time to compare expiration of the access token
						$sCurrentTimestamp = time();

						if(isset($aToken['validUntil']))
						{
							// Change the valid until ISO notation to UNIX timestamp
							$sExpirationTimestamp = strtotime($aToken['validUntil']);

							if($sCurrentTimestamp <= $sExpirationTimestamp)
							{
								return $aToken['token'];
							}
						}
					}
				}
				else
				{
					$sCacheFile = false;
				}
			}

			$sApiUrl = 'https://betalen.rabobank.nl/omnikassa-api' . ($this->getSandboxMode() ? '-sandbox' : '') . '/gatekeeper/refresh';

			$aArguments = array();
			$aArguments['body'] = '';
			$aArguments['timeout'] = '30';
			$aArguments['redirection'] = '5';
			$aArguments['httpversion'] = '1.1';
			$aArguments['blocking'] = true;
			$aArguments['headers'] = array('Expect' => '', 'Authorization'=> 'Bearer ' . $this->getRefreshToken());
			$aArguments['cookies'] = array();

			$oResponse = wp_remote_get($sApiUrl, $aArguments);
			$sResponse = wp_remote_retrieve_body($oResponse);

			if(!empty($sResponse))
			{
				$aToken = json_decode($sResponse, true);

				if(is_array($aToken) && sizeof($aToken))
				{
					if(!empty($aToken['errorCode']) && !empty($aToken['errorMessage']))
					{
						return '';
					}
					else
					{
						// Save data in cache?
						if($sCacheFile)
						{
							file_put_contents($sCacheFile, json_encode($aToken, JSON_PRETTY_PRINT));
						}

						return $aToken['token'];
					}
				}
				else
				{
					if($this->getSandboxMode())
					{
						wc_print_notice('Invalid response received from Rabo OmniKassa', 'error');
					}
				}
			}
			else
			{
				wc_print_notice('Accesstoken could not be generated, please check the configuration.', 'error');
			}

			wc_print_notice('Accesstoken could not be generated, please check the configuration.', 'error');
		}

		// Get issuers from Rabo OmniKassa or Cache
		public function getIssuers()
		{
			$aIssuers = array();
			$sCacheFile = false;
			$sCachePath = ICWOOROK_ROOT_PATH . 'temp' . DIRECTORY_SEPARATOR . 'cache' . DIRECTORY_SEPARATOR;

			// Used cached access token?
			// if(!$this->getSandboxMode() && $sCachePath)
			if($sCachePath)
			{
				$sStoreHost = md5($_SERVER['SERVER_NAME']);
				$sCacheFile = $sCachePath . 'issuers.' . $sStoreHost . '.cache';

				if(!file_exists($sCacheFile))
				{
					// Attempt to create cache file
					if(@touch($sCacheFile))
					{
						@chmod($sCacheFile, 0777);
					}
				}
				elseif(is_readable($sCacheFile) && is_writable($sCacheFile))
				{
					// Read data from cache file
					if($sData = file_get_contents($sCacheFile))
					{
						$aIssuers = json_decode($sData, true);

						// Get current time to compare expiration of the access token
						$sCurrentTimestamp = time();

						if(isset($aIssuers['validUntil']))
						{
							// Change the valid until ISO notation to UNIX timestamp
							$sExpirationTimestamp = strtotime($aIssuers['validUntil']);

							if($sCurrentTimestamp <= $sExpirationTimestamp)
							{
								return $aIssuers;
							}
						}
					}
				}
				else
				{
					$sCacheFile = false;
				}
			}

			// URL has to be changed
			$sApiUrl = 'https://betalen.rabobank.nl/omnikassa-api' . ($this->getSandboxMode() ? '-sandbox' : '') . '/ideal/server/api/v2/issuers';

			$aArguments = array();
			$aArguments['body'] = '';
			$aArguments['timeout'] = '30';
			$aArguments['redirection'] = '5';
			$aArguments['httpversion'] = '1.1';
			$aArguments['blocking'] = true;
			$aArguments['headers'] = array('Expect' => '', 'Authorization' => 'Bearer ' . $this->getAccessToken());
			$aArguments['cookies'] = array();

			$oResponse = wp_remote_get($sApiUrl, $aArguments);
			$sResponse = wp_remote_retrieve_body($oResponse);

			if(!empty($sResponse))
			{
				$aIssuers = json_decode($sResponse, true);

				if(is_array($aIssuers) && sizeof($aIssuers))
				{
					// Error received, show cache anyway
					if(!empty($aIssuers['errorCode']) && !empty($aIssuers['errorMessage']))
					{
						// Continue
					}
					else
					{
						// Save data in cache?
						if($sCacheFile)
						{
							$aIssuers['validUntil'] = date('c', (time() + ((60 * 60) * 24)));
						}
					}
				}

				file_put_contents($sCacheFile, json_encode($aIssuers, JSON_PRETTY_PRINT));
			}

			return $aIssuers;
		}

		public function getSimpleIssuers()
		{
			$aIssuers = $this->getIssuers();
			$aSimpleIssuers = array();

			foreach($aIssuers['issuers'] as $aIssuer)
			{
				$aSimpleIssuers[$aIssuer['id']] = $aIssuer['name'];
			}

			return $aSimpleIssuers;
		}

		public function getIcon()
		{
			$sGatewayCode = $this->getPaymentCode();

			if(file_exists(ICWOOROK_ROOT_PATH . 'images' . DIRECTORY_SEPARATOR . $sGatewayCode . '.svg'))
			{
				return plugins_url('images/' . $sGatewayCode . '.svg', dirname(__FILE__));
			}
			elseif(file_exists(ICWOOROK_ROOT_PATH . 'images' . DIRECTORY_SEPARATOR . $sGatewayCode . '.png'))
			{
				return plugins_url('images/' . $sGatewayCode . '.png', dirname(__FILE__));
			}
			else
			{
				return '';
			}
		}

		public function getLabel()
		{
			return __(' via Rabo OmniKassa 2.0', 'ic-woo-rabo-omnikassa-2');
		}

		public function getPaymentCode()
		{
			throw new Exception('Forgot the getPaymentCode method for this payment method?');
		}

		public function getPaymentName()
		{
			throw new Exception('Forgot the getPaymentName method for this payment method?');
		}

		public function isRedirect()
		{
			return true;
		}

		public function getRefreshToken()
		{
			return get_option('icwoorok2_refresh_token');
		}

		public function getSandboxMode()
		{
			$sSandboxMode = get_option('icwoorok2_sandbox');

			if(strcmp($sSandboxMode, 'yes') === 0)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		public function getSigningKey()
		{
			return get_option('icwoorok2_signing_key');
		}

		public function getIssuerOption()
		{
			$iIssuerOption = get_option('icwoorok2_show_issuers');

			return (int)$iIssuerOption;
		}

		/* Class functions */
		public function init_form_fields()
		{
			$this->form_fields = array();

			$this->form_fields['docs'] = array(
				'title'       => __('Rabo OmniKassa 2.0 Documentation', 'ic-woo-rabo-omnikassa-2'),
				'type'        => 'title',
				'description' => '<a href="https://www.rabobank.nl/bedrijven/betalen/klanten-laten-betalen/rabo-omnikassa-2-0/support-overzicht/rabo-onlinekassa-support/" target="_blank">' . __('Go to documentation', 'ic-woo-rabo-omnikassa-2') . '</a>.'
				);

			$this->form_fields['enabled'] = array(
					'title' => __('Enable/Disable', 'ic-woo-rabo-omnikassa-2'),
					'type' => 'checkbox',
					'label' => __('Enable Rabo OmniKassa', 'ic-woo-rabo-omnikassa-2') . ' - ' . $this->getPaymentName(),
					'default' => 'no'
				);

			$this->form_fields['title'] = array(
					'title' => __('Title', 'ic-woo-rabo-omnikassa-2'),
					'type' => 'text',
					'description' => __('This controls the title which the user sees during checkout.', 'ic-woo-rabo-omnikassa-2'),
					'default' => $this->getPaymentName(),
					'desc_tip' => true,
				);

			$this->form_fields['description'] = array(
					'title' => __('Customer Message after payment select', 'ic-woo-rabo-omnikassa-2'),
					'type' => 'textarea',
					'default' => 'Pay with ' . $this->getPaymentName(),
				);

			$this->form_fields['description_bankaccount'] = array(
					'title' => __('Description on receipt', 'ic-woo-rabo-omnikassa-2'),
					'type' => 'textarea',
					'default' => '',
				);
		}

		public function payment_fields()
		{
			$sPaymentDescription = $this->get_option('description');

			echo $sPaymentDescription;
		}

		public function process_payment($sOrderId)
		{
			global $wpdb;
			global $woocommerce;

			$oOrder = wc_get_order($sOrderId);

			if(version_compare($woocommerce->version, '3.0', '>='))
			{
				// Get all order related data
				$aOrderData = $oOrder->get_data();

				$sOrderNumber = $aOrderData['number'];
				$sDescription = $this->get_option('description_bankaccount');

				// Bank charge description empty, fallback
				if(empty($sDescription))
				{
					$sDescription = 'Bestelling ' . $sOrderNumber;
				}

				// Order amount in Cents
				$fOrderAmount = round($aOrderData['total'] * 100);
				$sCurrencyCode = $aOrderData['currency'];

				if(!in_array($sCurrencyCode, array('EUR')))
				{
					// Currency is not EUR, fallback to EUR and log?
					$sCurrencyCode = 'EUR';
				}

				$sLanguageCode = get_bloginfo('language');

				if(!in_array($sLanguageCode, array('nl', 'en', 'fr', 'de')))
				{
					$sLanguageCode = 'en';
				}
			}
			elseif(version_compare($woocommerce->version, '2.0', '>='))
			{
				$sOrderNumber = $oOrder->get_order_number();
				$sDescription = $this->get_option('description_bankaccount');

				// Bank charge description empty, fallback
				if(empty($sDescription))
				{
					$sDescription = 'Bestelling ' . $sOrderNumber;
				}

				// Order amount in Cents
				$fOrderAmount = round($oOrder->get_total() * 100);
				$sCurrencyCode = $oOrder->order_currency;

				if(!in_array($sCurrencyCode, array('EUR')))
				{
					// Currency is not EUR, fallback to EUR and log?
					$sCurrencyCode = 'EUR';
				}

				$sLanguageCode = get_bloginfo('language');

				if(!in_array($sLanguageCode, array('nl', 'en', 'fr', 'de')))
				{
					$sLanguageCode = 'en';
				}
			}

			$sAccessToken = $this->getAccessToken();

			$sReturnUrl = add_query_arg(array(
				'wc-api' => 'Wc_Omnikassa_Gateway_Return',
				'real_id' => $sOrderId,
			), home_url('/'));

			$sCancelUrl = $oOrder->get_cancel_order_url();
			$sNotifyUrl = add_query_arg('wc-api', 'Wc_Omnikassa_Gateway_Notify', home_url('/'));

			// Setup message for the order announcement
			$aRequest['timestamp'] = date('c', time());
			$aRequest['merchantOrderId'] = $oOrder->get_id();
			$aRequest['amount'] = array();
			$aRequest['amount']['currency'] = $sCurrencyCode;
			$aRequest['amount']['amount'] = $fOrderAmount;
			$aRequest['language'] = strtoupper($sLanguageCode);
			$aRequest['description'] = $sDescription;
			$aRequest['merchantReturnURL'] = $sReturnUrl;
			$aRequest['paymentBrand'] = strtoupper(substr($this->getPaymentCode(), 10));
			$aRequest['paymentBrandForce'] = 'FORCE_ALWAYS';

			
			// Customer Name
			$aRequest['customerInformation']['fullName'] = $aOrderData['billing']['first_name'] . ' ' . $aOrderData['billing']['last_name'];

			if(!$this->getSandboxMode())
			{
				$aRequest['skipHppResultPage'] = 'true';
			}

			// iDEAL specific data
			if($aRequest['paymentBrand'] == 'IDEAL')
			{
				if(array_key_exists($this->getPaymentCode() . '_issuer', $_POST))
				{
					$sIssuerId = sanitize_text_field($_POST[$this->getPaymentCode() . '_issuer']);

					$aRequest['paymentBrandMetaData'] = array();
					$aRequest['paymentBrandMetaData']['issuerId'] = $sIssuerId;
				}
			}

			// Afterpay specific data
			if($aRequest['paymentBrand'] == 'AFTERPAY')
			{
				$aOrderItems = $oOrder->get_items();
				$aProductData = array();

				foreach($aOrderItems as $k => $v)
				{
					$aProduct = array();

					$aProduct['id'] = $v['product_id'];
					$aProduct['name'] = substr($v['name'], 0, 25);
					$aProduct['description'] = substr($v['name'], 0, 100);

					$aProduct['quantity'] = intval($v['quantity']);

					$aProduct['amount'] = array();
					$aProduct['amount']['currency'] = 'EUR';
					$aProduct['amount']['amount'] = (round((($v['line_total'] + $v['line_tax']) / $v['quantity']), 2) * 100);

					// Tax rates
					$tax = '';
					$oTax = new WC_Tax();
					$oProduct = $oOrder->get_product_from_item($v);

                    foreach($oTax->get_shop_base_rate($oProduct->get_tax_class) as $aLineTax)
					{
                        $tax = $aLineTax['rate'];
                    }

					$aProduct['tax'] = array();
					$aProduct['tax']['currency'] = 'EUR';
					$aProduct['tax']['amount'] = (round(($v['line_tax'] / $v['quantity']), 2) * 100);

					$iVatCategory = '1';

					if(strcasecmp($tax, '21') === 0)
					{
						$iVatCategory = '1';
					}
					elseif(strcasecmp($tax, '9') === 0)
					{
						$iVatCategory = '2';
					}
					elseif(strcasecmp($tax, '0') === 0)
					{
						$iVatCategory = '3';
					}
					else
					{
						$iVatCategory = '4';
					}

					$aProduct['vatCategory'] = $iVatCategory;
					$aProduct['category'] = 'PHYSICAL';

					$aProductData[] = $aProduct;
				}


				//Shipping cost?
				$fShipping = $oOrder->get_shipping_total();

				if($fShipping > 0)
				{
					$aProduct = array();

					$aProduct['id'] = 'shipping';
					$aProduct['name'] = __('Shipment Cost', 'ic-woo-rabo-omnikassa-2');
					$aProduct['description'] = __('Cost of chosen shipment method', 'ic-woo-rabo-omnikassa-2');

					$aProduct['quantity'] = '1';

					$aProduct['amount'] = array();
					$aProduct['amount']['currency'] = 'EUR';
					$aProduct['amount']['amount'] = (round($oOrder->get_shipping_total() + $oOrder->get_shipping_tax(), 2) * 100);

					$aProduct['tax'] = array();
					$aProduct['tax']['currency'] = 'EUR';
					$aProduct['tax']['amount'] = (round($oOrder->get_shipping_tax(), 2) * 100);

					$aProduct['vatCategory'] = '1';
					$aProduct['category'] = 'PHYSICAL';

					$aProductData[] = $aProduct;
				}


				$iCount = 0;

				// Fees?
				foreach($oOrder->get_fees() as $oOrderFee)
				{
					$aProduct = array();

					$aProduct['id'] = 'Fee - ' . $iCount;
					$aProduct['name'] = $oOrderFee->get_name();
					$aProduct['description'] = $oOrderFee->get_name();

					$aProduct['quantity'] = $oOrderFee->get_quantity();

					$aProduct['amount'] = array();
					$aProduct['amount']['currency'] = 'EUR';
					$aProduct['amount']['amount'] = (round($oOrderFee->get_total() + $oOrderFee->get_total_tax(), 2) * 100);

					$aProduct['tax'] = array();
					$aProduct['tax']['currency'] = 'EUR';
					$aProduct['tax']['amount'] = (round($oOrderFee->get_total_tax(), 2) * 100);

					$taxrate = 0;

					$taxes = $oOrderFee->get_taxes();

					if(is_array($taxes) && is_array($taxes['total']) && count($taxes['total']) == 1)
					{
						$taxrate = (($taxes['total']['1'] * 100) / $oOrderFee->get_total());
					}

					$iVatCategory = '1';

					if(strcasecmp($taxrate, '21') === 0)
					{
						$iVatCategory = '1';
					}
					elseif(strcasecmp($taxrate, '9') === 0)
					{
						$iVatCategory = '2';
					}
					elseif(strcasecmp($taxrate, '0') === 0)
					{
						$iVatCategory = '3';
					}
					else
					{
						$iVatCategory = '4';
					}

					$aProduct['vatCategory'] = $iVatCategory;
					$aProduct['category'] = 'PHYSICAL';

					$aProductData[] = $aProduct;

					$iCount++;
				}

				$aShippingData = $aOrderData['shipping'];
				$aBillingData = $aOrderData['billing'];

				// Shipment data
				list($sStreetName, $sStreetNumber, $sExtension) = icwoorok2_mb_splitAddress($aShippingData['address_1'] . ' ' . $aShippingData['address_2']);

				$aCustomerShippingData['firstName'] = substr($aShippingData['first_name'], 0, 20);
				$aCustomerShippingData['middleName'] = '';
				$aCustomerShippingData['lastName'] = substr($aShippingData['last_name'], 0, 50);
				$aCustomerShippingData['street'] = substr($sStreetName, 0, 100);
				$aCustomerShippingData['houseNumber'] = substr($sStreetNumber, 0, 5);
				$aCustomerShippingData['houseNumberAddition'] = $sExtension;
				$aCustomerShippingData['postalCode'] = substr($aShippingData['postcode'], 0, 10);
				$aCustomerShippingData['city'] = substr($aShippingData['city'], 0, 40);
				$aCustomerShippingData['countryCode'] = substr($aShippingData['country'], 0, 2);


				list($sStreetName, $sStreetNumber, $sExtension) = icwoorok2_mb_splitAddress($aBillingData['address_1'] . ' ' . $aBillingData['address_2']);

				// Billing data
				$aPaymentCustomerData['firstName'] = substr($aBillingData['first_name'], 0, 20);
				$aPaymentCustomerData['middleName'] = '';
				$aPaymentCustomerData['lastName'] = substr($aBillingData['last_name'], 0, 50);
				$aPaymentCustomerData['street'] = substr($sStreetName, 0, 100);
				$aPaymentCustomerData['houseNumber'] = substr($sStreetNumber, 0, 5);
				$aPaymentCustomerData['houseNumberAddition'] = $sExtension;
				$aPaymentCustomerData['postalCode'] = substr($aBillingData['postcode'], 0, 10);
				$aPaymentCustomerData['city'] = substr($aBillingData['city'], 0, 40);
				$aPaymentCustomerData['countryCode'] = substr($aBillingData['country'], 0, 2);

				// Additional data
				$sGender = '';
				$sBirthDate = '';
				$sInitials = '';

				if(array_key_exists($this->getPaymentCode() . '_gender', $_POST))
				{
					$sGender = sanitize_text_field($_POST[$this->getPaymentCode() . '_gender']);
				}

				if(array_key_exists($this->getPaymentCode() . '_birthday_day', $_POST) && array_key_exists($this->getPaymentCode() . '_birthday_month', $_POST) && array_key_exists($this->getPaymentCode() . '_birthday_year', $_POST))
				{
					if(checkdate($_POST[$this->getPaymentCode() . '_birthday_month'], $_POST[$this->getPaymentCode() . '_birthday_month'], $_POST[$this->getPaymentCode() . '_birthday_year']))
					{
						$sBirthDate = sanitize_text_field($_POST[$this->getPaymentCode() . '_birthday_day']) . '-' . sanitize_text_field($_POST[$this->getPaymentCode() .
						'_birthday_month']) . '-' . sanitize_text_field($_POST[$this->getPaymentCode() . '_birthday_year']);
					}
				}

				$aAdditionalCustomerData['emailAddress'] = sanitize_email($oOrder->get_billing_email());
				$aAdditionalCustomerData['dateOfBirth'] = $sBirthDate;
				$aAdditionalCustomerData['gender'] = strtoupper($sGender);

				// Get initials
				// Replace all non alphabetic characters for a hyphen
				$sFirstname = preg_replace('/([^A-Za-z]+)/', '-', $aShippingData['first_name']);
				$aFirstnameParts = explode('-', $sFirstname);

				$sInitials = '';

				foreach($aFirstnameParts as $aFirstname)
				{
					$sInitials .= strtoupper(substr($aFirstname, 0, 1));
				}

				$aAdditionalCustomerData['initials'] = $sInitials;
				$aAdditionalCustomerData['telephoneNumber'] = $oOrder->get_billing_phone();

				$aRequest['orderItems'] = $aProductData;
				$aRequest['shippingDetail'] = $aCustomerShippingData;
				$aRequest['billingDetail'] = $aPaymentCustomerData;
				$aRequest['customerInformation'] = $aAdditionalCustomerData;
			}

			$sApiUrl = 'https://betalen.rabobank.nl/omnikassa-api' . ($this->getSandboxMode() ? '-sandbox' : '') . '/order/server/api/v2/order';

			$aArguments = array();
			$aArguments['body'] = json_encode($aRequest);
			$aArguments['timeout'] = '10';
			$aArguments['redirection'] = '5';
			$aArguments['httpversion'] = '1.1';
			$aArguments['blocking'] = true;
			$aArguments['headers'] = array('Expect' => '', 'Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $sAccessToken);
			$aArguments['cookies'] = array();


			$oResponse = wp_remote_post($sApiUrl, $aArguments);
			$sResponse = wp_remote_retrieve_body($oResponse);

			if(!empty($sResponse))
			{
				$aTransaction = json_decode($sResponse, true);

				if(is_array($aTransaction) && !empty($aTransaction))
				{
					if(isset($aTransaction['omnikassaOrderId']) && isset($aTransaction['redirectUrl']))
					{
						// Add note for chosen method:
						$oOrder->add_order_note(__('Rabo OmniKassa payment started with:', 'ic-woo-rabo-omnikassa-2') . $this->getPaymentName() . '<br>' . __('Using transaction ID:', 'ic-woo-rabo-omnikassa-2') . '<br>' . $aTransaction['omnikassaOrderId']);
						$oOrder->save();

						if($this->isRedirect())
						{
							return array('result' => 'success', 'redirect' => $aTransaction['redirectUrl']);
						}
						else
						{
							// Return thankyou redirect
							return array(
								'result' 	=> 'success',
								'redirect'	=> $this->get_return_url($oOrder)
							);
						}
					}
					elseif(isset($aTransaction['errorCode']))
					{
						$sErrorCode = $aTransaction['errorCode'];
						$sErrorMessage = $aTransaction['errorMessage'];

						// $oOrder->cancel_order('Rabo OmniKassa 2.0 fout: ' . $sErrorCode. ' : ' . $sErrorMessage);
						$oOrder->add_order_note(__('OmniKassa returned an error! Code:', 'ic-woo-rabo-omnikassa-2') . $sErrorCode . ' with message: <br>' . $sErrorMessage . '<br>' . __('Check our FAQ:', 'ic-woo-rabo-omnikassa-2') . ' <a href="https://www.ideal-checkout.nl/faq-ic/payment-providers/rabo-omnikassa-2-0/rok-2-error-codes" target="_blank">' . __('Rabo OmniKassa 2.0 Error codes', 'ic-woo-rabo-omnikassa-2') . '</a>');
					}
				}
				else
				{
					wc_print_notice('Order announcement could not be decoded, something wrong with the data received?', 'error');
				}
			}
			else
			{
				wc_print_notice('No response received from the Rabobank.', 'error');
			}
		}

	}


?>